@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2023 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -94,6 +94,7 @@ struct _RemminaFileEditorPriv {
 	const gchar *		avahi_service_type;
 
 	GtkWidget *		name_entry;
+	GtkWidget *		labels_entry;
 	GtkWidget *		group_combo;
 	GtkWidget *		protocol_combo;
 	GtkWidget *		save_button;
@@ -1404,7 +1405,7 @@ static void remmina_file_editor_save_ssh_tunnel_tab(RemminaFileEditor *gfe)
 			      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->ssh_tunnel_loopback_check)) :
 			      FALSE));
 	remmina_file_set_int(priv->remmina_file, "ssh_tunnel_enabled", ssh_tunnel_enabled);
-	remmina_file_set_string_ref(priv->remmina_file, "ssh_tunnel_auth",
+	remmina_file_set_string(priv->remmina_file, "ssh_tunnel_auth",
 			remmina_public_combo_get_active_text(GTK_COMBO_BOX(priv->ssh_tunnel_auth_combo)));
 	remmina_file_set_string(priv->remmina_file, "ssh_tunnel_username",
 				(ssh_tunnel_enabled ? gtk_entry_get_text(GTK_ENTRY(priv->ssh_tunnel_username_entry)) : NULL));
@@ -1587,7 +1588,7 @@ static GError* remmina_file_editor_update_settings(RemminaFileEditor *gfe,
 				break;
 			}
 
-			remmina_file_set_string_ref(priv->remmina_file, (gchar *)key, value);
+			remmina_file_set_string(priv->remmina_file, (gchar *)key, value);
 		} else if (GTK_IS_FILE_CHOOSER(widget)) {
 			gchar *value = gtk_widget_get_sensitive(GTK_WIDGET(widget)) ?
 								  gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget)) :
@@ -1596,11 +1597,13 @@ static GError* remmina_file_editor_update_settings(RemminaFileEditor *gfe,
 			if (!remmina_file_editor_validate_settings(gfe, (gchar*) key, value, &err)) {
 				// Error while validating!
 				// err should be set now.
+				g_free(value);
 				*failed_widget = widget;
 				break;
 			}
 
 			remmina_file_set_string(priv->remmina_file,	(gchar *)key, value);
+			g_free(value);
 		} else if (GTK_IS_TOGGLE_BUTTON(widget)) {
 			gboolean value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
@@ -1634,13 +1637,15 @@ static GError* remmina_file_editor_update(RemminaFileEditor *gfe,
 
 	remmina_file_set_string(priv->remmina_file, "name", gtk_entry_get_text(GTK_ENTRY(priv->name_entry)));
 
-	remmina_file_set_string_ref(priv->remmina_file, "group",
+	remmina_file_set_string(priv->remmina_file, "labels", gtk_entry_get_text(GTK_ENTRY(priv->labels_entry)));
+
+	remmina_file_set_string(priv->remmina_file, "group",
 				    (priv->group_combo ? remmina_public_combo_get_active_text(GTK_COMBO_BOX(priv->group_combo)) : NULL));
 
-	remmina_file_set_string_ref(priv->remmina_file, "protocol",
+	remmina_file_set_string(priv->remmina_file, "protocol",
 				    remmina_public_combo_get_active_text(GTK_COMBO_BOX(priv->protocol_combo)));
 
-	remmina_file_set_string_ref(priv->remmina_file, "server",
+	remmina_file_set_string(priv->remmina_file, "server",
 				    (priv->server_combo ? remmina_public_combo_get_active_text(GTK_COMBO_BOX(priv->server_combo)) : NULL));
 
 	if (priv->resolution_auto_radio) {
@@ -1667,7 +1672,7 @@ static GError* remmina_file_editor_update(RemminaFileEditor *gfe,
 	}
 
 	if (priv->keymap_combo)
-		remmina_file_set_string_ref(priv->remmina_file, "keymap",
+		remmina_file_set_string(priv->remmina_file, "keymap",
 					    remmina_public_combo_get_active_text(GTK_COMBO_BOX(priv->keymap_combo)));
 
 	remmina_file_editor_save_behavior_tab(gfe);
@@ -1924,6 +1929,8 @@ GtkWidget *remmina_file_editor_new_from_file(RemminaFile *remminafile)
 
 	//remmina_public_create_group(GTK_GRID(grid), _("Profile"), 0, 4, 3);
 
+	gboolean profile_file_exists = (remmina_file_get_filename(remminafile) != NULL);
+
 	/* Profile: Name */
 	widget = gtk_label_new(_("Name"));
 	gtk_widget_show(widget);
@@ -1938,7 +1945,7 @@ GtkWidget *remmina_file_editor_new_from_file(RemminaFile *remminafile)
 	gtk_entry_set_max_length(GTK_ENTRY(widget), 100);
 	priv->name_entry = widget;
 
-	if (remmina_file_get_filename(remminafile) == NULL) {
+	if (!profile_file_exists) {
 		gtk_entry_set_text(GTK_ENTRY(widget), _("Quick Connect"));
 #if GTK_CHECK_VERSION(3, 16, 0)
 		gtk_entry_grab_focus_without_selecting(GTK_ENTRY(widget));
@@ -1967,16 +1974,41 @@ GtkWidget *remmina_file_editor_new_from_file(RemminaFile *remminafile)
 	gtk_widget_set_tooltip_text(priv->group_combo, s);
 	g_free(s);
 
+	/* Profile: Labels */
+	widget = gtk_label_new(_("Labels"));
+	gtk_widget_show(widget);
+	gtk_widget_set_valign(widget, GTK_ALIGN_START);
+	gtk_widget_set_halign(widget, GTK_ALIGN_START);
+	gtk_grid_attach(GTK_GRID(grid), widget, 0, 9, 2, 1);
+	gtk_grid_set_column_spacing(GTK_GRID(grid), 10);
+
+	widget = gtk_entry_new();
+	gtk_widget_show(widget);
+	gtk_grid_attach(GTK_GRID(grid), widget, 1, 9, 3, 1);
+	gtk_entry_set_max_length(GTK_ENTRY(widget), 255);
+	priv->labels_entry = widget;
+
+	if (!profile_file_exists) {
+		gtk_widget_set_tooltip_text(widget, _("Label1,Label2"));
+#if GTK_CHECK_VERSION(3, 16, 0)
+		gtk_entry_grab_focus_without_selecting(GTK_ENTRY(widget));
+#endif
+		g_signal_connect(G_OBJECT(widget), "changed", G_CALLBACK(remmina_file_editor_entry_on_changed), gfe);
+	} else {
+		cs = remmina_file_get_string(remminafile, "labels");
+		gtk_entry_set_text(GTK_ENTRY(widget), cs ? cs : "");
+	}
+
 	/* Profile: Protocol */
 	widget = gtk_label_new(_("Protocol"));
 	gtk_widget_show(widget);
 	gtk_widget_set_valign(widget, GTK_ALIGN_START);
 	gtk_widget_set_halign(widget, GTK_ALIGN_START);
-	gtk_grid_attach(GTK_GRID(grid), widget, 0, 9, 2, 1);
+	gtk_grid_attach(GTK_GRID(grid), widget, 0, 12, 2, 1);
 
 	widget = remmina_public_create_combo(TRUE);
 	gtk_widget_show(widget);
-	gtk_grid_attach(GTK_GRID(grid), widget, 1, 9, 3, 1);
+	gtk_grid_attach(GTK_GRID(grid), widget, 1, 12, 3, 1);
 	priv->protocol_combo = widget;
 	remmina_plugin_manager_for_each_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, remmina_file_editor_iterate_protocol, gfe);
 	g_signal_connect(G_OBJECT(widget), "changed", G_CALLBACK(remmina_file_editor_protocol_combo_on_changed), gfe);
